@@ -1,5 +1,6 @@
 pub mod bpe;
 pub mod ctc;
+pub mod sequence;
 pub mod wordpiece;
 
 // Re-export these as decoders
@@ -10,6 +11,7 @@ use serde::{Deserialize, Serialize};
 
 use crate::decoders::bpe::BPEDecoder;
 use crate::decoders::ctc::CTC;
+use crate::decoders::sequence::Sequence;
 use crate::decoders::wordpiece::WordPiece;
 use crate::pre_tokenizers::byte_level::ByteLevel;
 use crate::pre_tokenizers::metaspace::Metaspace;
@@ -23,16 +25,18 @@ pub enum DecoderWrapper {
     WordPiece(WordPiece),
     Metaspace(Metaspace),
     CTC(CTC),
+    Sequence(Sequence),
 }
 
 impl Decoder for DecoderWrapper {
-    fn decode(&self, tokens: Vec<String>) -> Result<String> {
+    fn decode_chain(&self, tokens: Vec<String>) -> Result<Vec<String>> {
         match self {
-            Self::BPE(bpe) => bpe.decode(tokens),
-            Self::ByteLevel(bl) => bl.decode(tokens),
-            Self::Metaspace(ms) => ms.decode(tokens),
-            Self::WordPiece(wp) => wp.decode(tokens),
-            Self::CTC(ctc) => ctc.decode(tokens),
+            Self::BPE(bpe) => bpe.decode_chain(tokens),
+            Self::ByteLevel(bl) => bl.decode_chain(tokens),
+            Self::Metaspace(ms) => ms.decode_chain(tokens),
+            Self::WordPiece(wp) => wp.decode_chain(tokens),
+            Self::CTC(ctc) => ctc.decode_chain(tokens),
+            Self::Sequence(seq) => seq.decode_chain(tokens),
         }
     }
 }
@@ -42,3 +46,4 @@ impl_enum_from!(ByteLevel, DecoderWrapper, ByteLevel);
 impl_enum_from!(Metaspace, DecoderWrapper, Metaspace);
 impl_enum_from!(WordPiece, DecoderWrapper, WordPiece);
 impl_enum_from!(CTC, DecoderWrapper, CTC);
+impl_enum_from!(Sequence, DecoderWrapper, Sequence);
