@@ -209,7 +209,10 @@ impl UnigramTrainer {
                 }
             }
         }
+        #[cfg(feature = "esaxx_fast")]
         let suffix = esaxx_rs::suffix(&flat_string).unwrap();
+        #[cfg(not(feature = "esaxx_fast"))]
+        let suffix = esaxx_rs::suffix_rs(&flat_string).unwrap();
 
         //  Basic chars need to be in sentence pieces.
         let mut seed_sentencepieces: Vec<SentencePiece> = vec![];
