@@ -2,6 +2,7 @@ use super::{
     normalizer::Range, Model, NormalizedString, Normalizer, Offsets, PreTokenizedString, Token,
 };
 use aho_corasick::{AhoCorasick, AhoCorasickBuilder, MatchKind};
+use regex::Regex;
 use serde::{ser::SerializeSeq, Deserialize, Serialize, Serializer};
 use std::collections::{HashMap, HashSet};
 
@@ -22,15 +23,18 @@ pub struct AddedToken {
     pub rstrip: bool,
     /// Whether this token should be normalized
     pub normalized: bool,
+    /// Whether this token is special
+    pub special: bool,
 }
 
 impl AddedToken {
     /// Build this token from the given content, specifying if it is intented to be a
     /// special token. Special tokens are not normalized by default.
     pub fn from<S: Into<String>>(content: S, special: bool) -> Self {
-        AddedToken {
+        Self {
             content: content.into(),
             normalized: !special,
+            special,
             ..Default::default()
         }
     }
@@ -65,12 +69,13 @@ impl AddedToken {
 }
 impl Default for AddedToken {
     fn default() -> Self {
-        AddedToken {
+        Self {
             content: String::new(),
             single_word: false,
             lstrip: false,
             rstrip: false,
             normalized: true,
+            special: false,
         }
     }
 }
@@ -90,6 +95,35 @@ impl std::cmp::Eq for AddedToken {}
 
 type MatchingSet = (AhoCorasick, Vec<u32>);
 
+lazy_static! {
+    static ref STARTS_WITH_WORD: Regex = Regex::new(r"^\w").unwrap();
+    static ref ENDS_WITH_WORD: Regex = Regex::new(r"\w$").unwrap();
+    static ref RIGHTMOST_SPACE_AT_START: Regex = Regex::new(r"^\s*").unwrap();
+    static ref LEFTMOST_SPACE_AT_END: Regex = Regex::new(r"\s*$").unwrap();
+}
+
+fn ends_with_word(sentence: &str) -> bool {
+    ENDS_WITH_WORD.is_match(sentence)
+}
+
+fn starts_with_word(sentence: &str) -> bool {
+    STARTS_WITH_WORD.is_match(sentence)
+}
+
+fn space_leftmost_at_end(sentence: &str) -> usize {
+    if let Some(match_) = LEFTMOST_SPACE_AT_END.find(sentence) {
+        match_.start()
+    } else {
+        sentence.len()
+    }
+}
+fn space_rightmost_at_start(sentence: &str) -> usize {
+    if let Some(match_) = RIGHTMOST_SPACE_AT_START.find(sentence) {
+        match_.end()
+    } else {
+        0
+    }
+}
 ///
 /// A vocabulary built on top of the Model
 ///
@@ -186,13 +220,6 @@ impl AddedVocabulary {
         model: &impl Model,
         normalizer: Option<&N>,
     ) -> usize {
-        for token in tokens {
-            if !token.content.is_empty() && !self.special_tokens_set.contains(&token.content) {
-                self.special_tokens.push(token.to_owned());
-                self.special_tokens_set.insert(token.content.clone());
-            }
-        }
-        // Then we delegate to `add_tokens`, that will take care of refreshing added tokens too.
         self.add_tokens(tokens, model, normalizer)
     }
 
@@ -203,6 +230,18 @@ impl AddedVocabulary {
         model: &impl Model,
         normalizer: Option<&N>,
     ) -> usize {
+        // Handle special tokens (if any)
+        for token in tokens {
+            if token.special
+                && !token.content.is_empty()
+                && !self.special_tokens_set.contains(&token.content)
+            {
+                self.special_tokens.push(token.to_owned());
+                self.special_tokens_set.insert(token.content.clone());
+            }
+        }
+
+        // Then we delegate to `add_tokens`, that will take care of refreshing added tokens too.
         let mut ignored = 0;
         for token in tokens {
             if token.content.is_empty() {
@@ -302,9 +341,8 @@ impl AddedVocabulary {
             let id = split_re.1[aho_id];
             let added_token = &self.added_tokens_map_r.get(&id).unwrap();
             if added_token.single_word {
-                let start_space = start == 0 || sentence.as_bytes().get(start - 1) == Some(&b' ');
-                let stop_space =
-                    stop == sentence.len() || sentence.as_bytes().get(stop + 1) == Some(&b' ');
+                let start_space = start == 0 || !ends_with_word(&sentence[..start]);
+                let stop_space = stop == sentence.len() || !starts_with_word(&sentence[stop..]);
 
                 if !stop_space || !start_space {
                     // Discard not single word
@@ -312,22 +350,17 @@ impl AddedVocabulary {
                 }
             }
             if added_token.lstrip {
-                while start > 0 {
-                    if let Some(b' ') = sentence.as_bytes().get(start - 1) {
-                        start -= 1;
-                    } else {
-                        break;
-                    }
-                }
+                // This will be strictly inferior to start and in correct sentence offset
+                let newstart = space_leftmost_at_end(&sentence[..start]);
+
+                // The previous match could have already matched those spaces
+                // Ignore them if it's already matched
+                start = std::cmp::max(newstart, start_offset);
             }
             if added_token.rstrip {
-                while stop < sentence.len() {
-                    if let Some(b' ') = sentence.as_bytes().get(stop) {
-                        stop += 1;
-                    } else {
-                        break;
-                    }
-                }
+                // This will starting a the stop+1 character, so we need
+                // to add the previous stop value
+                stop += space_rightmost_at_start(&sentence[stop..])
             }
             if start_offset < start {
                 splits.push((None, (start_offset, start)));
@@ -403,9 +436,6 @@ impl AddedVocabulary {
 pub(super) struct AddedTokenWithId {
     /// The id assigned to this token
     pub id: u32,
-    /// Whether this is a special token
-    pub special: bool,
-
     #[serde(flatten)]
     /// The target AddedToken
     pub token: AddedToken,
@@ -421,7 +451,6 @@ impl Serialize for AddedVocabulary {
             .iter()
             .map(|(id, token)| AddedTokenWithId {
                 id: *id,
-                special: self.special_tokens_set.contains(&token.content),
                 token: token.clone(),
             })
             .collect::<Vec<_>>();
@@ -469,6 +498,21 @@ mod tests {
         }
     }
 
+    fn simplify_output(result: &'_ PreTokenizedString) -> Vec<(&'_ str, Option<Vec<u32>>)> {
+        result
+            .get_splits(OffsetReferential::Original, OffsetType::Byte)
+            .into_iter()
+            .map(|(s, _, tokens)| {
+                (
+                    s,
+                    tokens
+                        .as_ref()
+                        .map(|t| t.iter().map(|t| t.id).collect::<Vec<_>>()),
+                )
+            })
+            .collect::<Vec<_>>()
+    }
+
     struct TrainerMock;
     impl Trainer for TrainerMock {
         type Model = ModelMock;
@@ -669,16 +713,7 @@ mod tests {
             vocab.extract_and_normalize(Some(&normalizer), "[CLS] My name is Anthony [SEP]");
 
         assert_eq!(
-            result
-                .get_splits(OffsetReferential::Original, OffsetType::Byte)
-                .into_iter()
-                .map(|(s, _, tokens)| (
-                    s,
-                    tokens
-                        .as_ref()
-                        .map(|t| t.iter().map(|t| t.id).collect::<Vec<_>>())
-                ))
-                .collect::<Vec<_>>(),
+            simplify_output(&result),
             vec![
                 ("[CLS]", Some(vec![3])),
                 // This one includes both spaces because of the lstrip & rstrip
@@ -698,4 +733,92 @@ mod tests {
         let matches = vocab.find_matches("", &vocab.split_trie);
         assert_eq!(matches, vec![(None, (0, 0))]);
     }
+
+    #[test]
+    fn test_single_word_is_correct() {
+        // Is able to extract both normal and special tokens, with various options (lstrip, rstrip,
+        // single_word, normalized)
+        let model = ModelMock::new(&[]);
+        let mut vocab = AddedVocabulary::new();
+        let normalizer = Lowercase;
+
+        vocab.add_tokens(
+            &[AddedToken::from("<mask>", false).single_word(true)],
+            &model,
+            Some(&normalizer),
+        );
+        // Left, in the middle, non single world left, non single word right, end of sentence valid
+        let result = vocab.extract_and_normalize(
+            Some(&normalizer),
+            "<mask> My name <mask> A<mask> <mask>ony <mask>",
+        );
+        assert_eq!(
+            simplify_output(&result),
+            vec![
+                ("<mask>", Some(vec![0])),
+                (" my name ", None),
+                ("<mask>", Some(vec![0])),
+                (" a<mask> <mask>ony ", None),
+                ("<mask>", Some(vec![0]))
+            ]
+        );
+    }
+
+    #[test]
+    fn test_single_word_is_unicode_correct() {
+        let model = ModelMock::new(&[]);
+        let mut vocab = AddedVocabulary::new();
+        let normalizer = Lowercase;
+
+        vocab.add_tokens(
+            &[AddedToken::from("<mask>", false).single_word(true)],
+            &model,
+            Some(&normalizer),
+        );
+        let result = vocab.extract_and_normalize(Some(&normalizer), "<mask>, <mask>- ◌̰<mask>");
+        assert_eq!(
+            simplify_output(&result),
+            vec![
+                // Punctuation is not word
+                ("<mask>", Some(vec![0])),
+                (", ", None),
+                // dash is not word
+                ("<mask>", Some(vec![0])),
+                // This is unicode combining mark character and is word: https://en.wikipedia.org/wiki/Combining_Diacritical_Marks
+                ("- ◌̰<mask>", None),
+            ]
+        );
+    }
+
+    #[test]
+    fn test_lstrip_unicode_space() {
+        let model = ModelMock::new(&[]);
+        let mut vocab = AddedVocabulary::new();
+        let normalizer = Lowercase;
+
+        vocab.add_tokens(
+            &[AddedToken::from("<mask>", false)
+                .lstrip(true)
+                .rstrip(true)
+                .single_word(true)],
+            &model,
+            Some(&normalizer),
+        );
+        let result = vocab
+            .extract_and_normalize(Some(&normalizer), "Hi <mask> there\t<mask>\t<mask>\u{2000}");
+        assert_eq!(
+            simplify_output(&result),
+            vec![
+                ("hi", None),
+                // Regular space
+                (" <mask> ", Some(vec![0])),
+                ("there", None),
+                // \t is a spacing character
+                ("\t<mask>\t", Some(vec![0])),
+                // Non overlapping
+                // \u{2000} is mongolian vowel separator: https://jkorpela.fi/chars/spaces.html
+                ("<mask>\u{2000}", Some(vec![0])),
+            ]
+        );
+    }
 }
